//----------------------------------------------------------------------------
// ObjectWindows - (C) Copyright 1991, 1993 by Borland International
//   Arty demo window object
//----------------------------------------------------------------------------
#include <owl\owlpch.h>
#include <owl\dc.h>
#include <owl\static.h>
#include <stdlib.h>
#include <time.h>
#include "artypriv.h"       // internal component classes of TArtyWindow
#include "arty.h"           // class definition for TArtyWindow


//---------- TList ----------------------------------------------

// The low-level line draw routine
//
static void
LineDraw(TDC& dc, int x1, int y1, int x2, int y2)
{
  dc.MoveTo(x1, y1);
  dc.LineTo(x2, y2);
}

// Initialize the list-of-lines object 
//
TList::TList(int _max)
{
  MaxLines = min(_max,(int) MaxLineCount);
  CurrentLine = 1;
  Xmax = 0;
  Ymax = 0;
  ColorDuration = MaxColorDuration;
  IncrementCount = 0;
  MaxDelta = 10;
  randomize();
  PenColor = TColor(random(256), random(256), random(256));
}

// Keep X within range, and reverse Delta if necessary to do so
//
void
TList::AdjustX(int& x, int& deltaX)
{
  int testX = x + deltaX;
  if (testX < 1 || testX > Xmax) {
    testX = x;
    deltaX = -deltaX;
  }
  x = testX;
}

// Keep Y within range, and reverse Delta if necessary to do so
//
void
TList::AdjustY(int& y, int& deltaY)
{
  int testY = y + deltaY;
  if (testY < 1 || testY > Ymax) {
    testY = y;
    deltaY = -deltaY;
  }
  y = testY;
}

// Clear the array of lines
//
void
TList::ResetLines()
{
  int startX = random(Xmax / 2);
  int startY = random(Ymax / 2);
  for (int i = 0; i < MaxLines; i++) {
	 Line[i].LX1 = startX;
	 Line[i].LX2 = startX;
	 Line[i].LY1 = startY;
	 Line[i].LY2 = startY;
	 Line[i].Color = 0;
  }
  X1 = startX;
  X2 = startX;
  Y1 = startY;
  Y2 = startY;
}

// Scale the old line coordinates to the new Xmax and Ymax coordinates.
//  The new Xmax and new Ymax are passed in as parameters so we can
//  calculate the scaling ratios.
//
void
TList::ScaleTo(int newXmax, int newYmax)
{
  if (!Xmax || !Ymax) {    // at startup, Xmax and Ymax are zero
    Xmax = newXmax;
    Ymax = newYmax;
    ResetLines();

  } else {
    X1 = int(long(X1)*newXmax / Xmax);
    Y1 = int(long(Y1)*newYmax / Ymax);
    X2 = int(long(X2)*newXmax / Xmax);
    Y2 = int(long(Y2)*newYmax / Ymax);
    for (int i = 0; i < MaxLines; i++) {
      Line[i].LX1 = int(long(Line[i].LX1)*newXmax / Xmax);
      Line[i].LX2 = int(long(Line[i].LX2)*newXmax / Xmax);
      Line[i].LY1 = int(long(Line[i].LY1)*newYmax / Ymax);
      Line[i].LY2 = int(long(Line[i].LY2)*newYmax / Ymax);
    }
  }
  Xmax = newXmax;
  Ymax = newYmax;
}

// The high-level Draw method of the object.
//
void
TList::DrawLine(TDC& dc, int index)
{
  TPen pen(Line[index].Color);
  dc.SelectObject(pen);
  LineDraw(dc, Line[index].LX1, Line[index].LY1,
               Line[index].LX2, Line[index].LY2);
  dc.RestorePen();
}

// The high-level draw which erases a line.
//
void TList::EraseLine(TDC& dc, int index)
{
  dc.SelectStockObject(BLACK_PEN);
  LineDraw(dc, Line[index].LX1, Line[index].LY1,
               Line[index].LX2, Line[index].LY2);
}

// Redraw all the lines in the array.
//
void
TList::Redraw(TDC& dc)
{
  for (int i = 0; i < MaxLines; i++)
    DrawLine(dc, i);
}

// Reset the color counter and pick a random color.
//
void
TList::SelectNewColor()
{
  ColorDuration = MaxColorDuration;
  PenColor = TColor(random(256), random(256), random(256));
}

// Pick random directional deltas and reset the delta counter.
//
void
TList::SelectNewDeltaValues()
{
  DeltaX1 = random(MaxDelta) - MaxDelta/2;
  DeltaX2 = random(MaxDelta) - MaxDelta/2;
  DeltaY1 = random(MaxDelta) - MaxDelta/2;
  DeltaY2 = random(MaxDelta) - MaxDelta/2;
  IncrementCount = 2 * (1 + random(10));
}

// Process the movement of one line.
//
void
TList::LineTick(TDC& dc)
{
  EraseLine(dc, CurrentLine);
  if (ColorDuration < 0)
	 SelectNewColor();
  if (!IncrementCount)
	 SelectNewDeltaValues();
  AdjustX(X1, DeltaX1);
  AdjustX(X2, DeltaX2);
  AdjustY(Y1, DeltaY1);
  AdjustY(Y2, DeltaY2);

  Line[CurrentLine].LX1 = X1;
  Line[CurrentLine].LX2 = X2;
  Line[CurrentLine].LY1 = Y1;
  Line[CurrentLine].LY2 = Y2;
  Line[CurrentLine].Color = PenColor;

  DrawLine(dc, CurrentLine);
  CurrentLine++;
  if (CurrentLine >= MaxLines)
	 CurrentLine = 1;
  ColorDuration--;
  IncrementCount--;
}

//------------ TQuadList ----------------------------------------

// Draw the line and 3 reflections of it.
//
void
TQuadList::DrawLine(TDC& dc, int index)
{
  TPen pen(Line[index].Color);
  dc.SelectObject(pen);
  LineDraw(dc, Line[index].LX1, Line[index].LY1,
					Line[index].LX2, Line[index].LY2);
  LineDraw(dc, Xmax - Line[index].LX1, Line[index].LY1,
					Xmax - Line[index].LX2, Line[index].LY2);
  LineDraw(dc, Line[index].LX1, Ymax - Line[index].LY1,
					Line[index].LX2, Ymax - Line[index].LY2);
  LineDraw(dc, Xmax - Line[index].LX1, Ymax - Line[index].LY1,
               Xmax - Line[index].LX2, Ymax - Line[index].LY2);
  dc.RestorePen();
}

// Erase the line and 3 reflections of it.
//
void
TQuadList::EraseLine(TDC& dc, int index)
{
  dc.SelectStockObject(BLACK_PEN);
  LineDraw(dc, Line[index].LX1, Line[index].LY1,
               Line[index].LX2, Line[index].LY2);
  LineDraw(dc, Xmax - Line[index].LX1, Line[index].LY1,
               Xmax - Line[index].LX2, Line[index].LY2);
  LineDraw(dc, Line[index].LX1, Ymax - Line[index].LY1,
					Line[index].LX2, Ymax - Line[index].LY2);
  LineDraw(dc, Xmax - Line[index].LX1, Ymax - Line[index].LY1,
					Xmax - Line[index].LX2, Ymax - Line[index].LY2);
}

//----------- TArtyWindow ------------------------------------------

TArtyWindow::TArtyWindow(TWindow* parent, const char* title,
								 TModule* module)
  : TWindow(parent, title, module),
	 TScrnSavWindow(parent, title, module)
{
  TextHeight = 20;

  ScreenSize.cx = GetSystemMetrics(SM_CXSCREEN);
  ScreenSize.cy = GetSystemMetrics(SM_CYSCREEN);

  List = new TQuadList(MaxLineCount);
  List->ScaleTo(ScreenSize.cx, ScreenSize.cy);
  List->ResetLines();
}

TArtyWindow::~TArtyWindow()
{
  delete List;
}

void
TArtyWindow::GetWindowClass(WNDCLASS& wndClass)
{
  TScrnSavWindow::GetWindowClass(wndClass);
  wndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
}

void
TArtyWindow::AnimateScreen()
{
/*
  TClientDC  dc(*this);

  List->LineTick(dc);
  List->Redraw(dc);
*/
}

void
TArtyWindow::TimerTick()
{
  TClientDC  dc(*this);

  List->LineTick(dc);
}

